import { ILayoutRestorer } from '@jupyterlab/application';
import { ICommandPalette, WidgetTracker, ToolbarButton } from '@jupyterlab/apputils';
import { PageConfig } from '@jupyterlab/coreutils';
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { IMainMenu } from '@jupyterlab/mainmenu';
import { INotebookTracker } from '@jupyterlab/notebook';
import classicRender from '@datalayer/icons-react/data1/DashboardColorIconLabIcon';
import { IClassicRenderTracker, ClassicRenderFactory } from './classicRender';
export var CommandIDs;
(function (CommandIDs) {
    CommandIDs.classicRender = 'notebook:render-with-classic';
    CommandIDs.classicOpen = 'notebook:open-with-classic';
})(CommandIDs || (CommandIDs = {}));
export const CLASSIC_RENDER_WIDGET_FACTORY = 'Classic Render';
class ClassicRenderButton {
    _commands;
    constructor(commands) {
        this._commands = commands;
    }
    createNew(panel) {
        const button = new ToolbarButton({
            className: 'classicRender',
            tooltip: 'Render with Classic',
            icon: classicRender,
            onClick: () => { this._commands.execute(CommandIDs.classicRender); }
        });
        panel.toolbar.insertAfter('cellType', 'classicRender', button);
        return button;
    }
}
/**
 * Initialization data for the jupyterlab-preview extension.
 */
const notebookClassicPlugin = {
    id: '@datalayer/jupyter-react:classic',
    autoStart: true,
    requires: [INotebookTracker],
    optional: [ICommandPalette, ILayoutRestorer, IMainMenu, ISettingRegistry],
    provides: IClassicRenderTracker,
    activate: (app, notebookTracker, palette, restorer, menu, settingRegistry) => {
        const { commands, docRegistry } = app;
        const tracker = new WidgetTracker({
            namespace: 'classic-preview'
        });
        if (restorer) {
            restorer.restore(tracker, {
                command: 'docmanager:open',
                args: panel => ({
                    path: panel.context.path,
                    factory: classicRenderFactory.name
                }),
                name: panel => panel.context.path,
                when: app.serviceManager.ready
            });
        }
        function getCurrentNotebookPanel(args) {
            const notebookPanel = notebookTracker.currentWidget;
            const activate = args['activate'] !== false;
            if (activate && notebookPanel) {
                app.shell.activateById(notebookPanel.id);
            }
            return notebookPanel;
        }
        function isEnabled() {
            return (notebookTracker.currentWidget !== null &&
                notebookTracker.currentWidget === app.shell.currentWidget);
        }
        function getClassicUrl(path) {
            const baseUrl = PageConfig.getBaseUrl();
            return `${baseUrl}tree/${path}`;
        }
        const classicRenderFactory = new ClassicRenderFactory(getClassicUrl, {
            name: CLASSIC_RENDER_WIDGET_FACTORY,
            fileTypes: ['notebook'],
            modelName: 'notebook'
        });
        classicRenderFactory.widgetCreated.connect((sender, classicRender) => {
            classicRender.context.pathChanged.connect(() => {
                void tracker.save(classicRender);
            });
            void tracker.add(classicRender);
        });
        const updateSettings = (settings) => {
            classicRenderFactory.defaultRenderOnSave = settings.get('renderOnSave')
                .composite;
        };
        if (settingRegistry) {
            Promise.all([settingRegistry.load(notebookClassicPlugin.id), app.restored])
                .then(([settings]) => {
                updateSettings(settings);
                settings.changed.connect(updateSettings);
            })
                .catch((reason) => {
                console.error(reason.message);
            });
        }
        commands.addCommand(CommandIDs.classicRender, {
            label: 'Render Notebook with Classic',
            execute: async (args) => {
                const notebookPanel = getCurrentNotebookPanel(args);
                let context;
                if (notebookPanel) {
                    context = notebookPanel.context;
                    await context.save();
                    commands.execute('docmanager:open', {
                        path: context.path,
                        factory: CLASSIC_RENDER_WIDGET_FACTORY,
                        options: {
                            mode: 'split-right'
                        }
                    });
                }
            },
            isEnabled
        });
        commands.addCommand(CommandIDs.classicOpen, {
            label: 'Render with Classic',
            execute: async (args) => {
                const notebookPanel = getCurrentNotebookPanel(args);
                if (!notebookPanel) {
                    return;
                }
                await notebookPanel.context.save();
                const previewUrl = getClassicUrl(notebookPanel.context.path);
                window.open(previewUrl);
            },
            isEnabled
        });
        if (palette) {
            const category = 'Notebook Operations';
            [CommandIDs.classicRender, CommandIDs.classicOpen].forEach(command => {
                palette.addItem({ command, category });
            });
        }
        if (menu) {
            menu.viewMenu.addGroup([
                {
                    command: CommandIDs.classicRender
                },
                {
                    command: CommandIDs.classicOpen
                }
            ], 1000);
        }
        const classicRenderButton = new ClassicRenderButton(commands);
        //
        docRegistry.addWidgetFactory(classicRenderFactory);
        docRegistry.addWidgetExtension('Notebook', classicRenderButton);
        //
        return tracker;
    }
};
export default notebookClassicPlugin;
//# sourceMappingURL=plugin.js.map