import { ILayoutRestorer } from '@jupyterlab/application';
import { ICommandPalette, WidgetTracker, ToolbarButton } from '@jupyterlab/apputils';
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { IMainMenu } from '@jupyterlab/mainmenu';
import { INotebookTracker } from '@jupyterlab/notebook';
import dashboardIcon from '@datalayer/icons-react/data1/AcademicCapIconLabIcon';
import { IDashboardTracker, DashboardFactory } from './dashboard';
export var CommandIDs;
(function (CommandIDs) {
    CommandIDs.dashboardRender = 'notebook:render-with-dashboard';
    CommandIDs.dashboardOpen = 'notebook:open-with-dashboard';
})(CommandIDs || (CommandIDs = {}));
export const DASHBOARD_WIDGET_FACTORY = 'Dashboard Preview';
class DashboardButton {
    _commands;
    constructor(commands) {
        this._commands = commands;
    }
    createNew(panel) {
        const button = new ToolbarButton({
            className: 'dashboardRender',
            tooltip: 'Dashboard',
            icon: dashboardIcon,
            onClick: () => { this._commands.execute(CommandIDs.dashboardRender); }
        });
        panel.toolbar.insertAfter('cellType', 'dashboardRender', button);
        return button;
    }
}
/**
 * Initialization data for the jupyterlab-preview extension.
 */
const dashboardPlugin = {
    id: '@datalayer/jupyter-react:dashboard',
    autoStart: true,
    requires: [INotebookTracker],
    optional: [ICommandPalette, ILayoutRestorer, IMainMenu, ISettingRegistry],
    provides: IDashboardTracker,
    activate: (app, notebookTracker, palette, restorer, menu, settingRegistry) => {
        const { commands, docRegistry } = app;
        const tracker = new WidgetTracker({
            namespace: 'dashboard-preview'
        });
        if (restorer) {
            restorer.restore(tracker, {
                command: 'docmanager:open',
                args: panel => ({
                    path: panel.context.path,
                    factory: dashboardRenderFactory.name
                }),
                name: panel => panel.context.path,
                when: app.serviceManager.ready
            });
        }
        function getCurrentNotebookPanel(args) {
            const notebookPanel = notebookTracker.currentWidget;
            const activate = args['activate'] !== false;
            if (activate && notebookPanel) {
                app.shell.activateById(notebookPanel.id);
            }
            return notebookPanel;
        }
        function isEnabled() {
            return (notebookTracker.currentWidget !== null &&
                notebookTracker.currentWidget === app.shell.currentWidget);
        }
        const dashboardRenderFactory = new DashboardFactory({
            name: DASHBOARD_WIDGET_FACTORY,
            fileTypes: ['notebook'],
            modelName: 'notebook',
        });
        dashboardRenderFactory.widgetCreated.connect((sender, dashboard) => {
            dashboard.context.pathChanged.connect(() => {
                void tracker.save(dashboard);
            });
            void tracker.add(dashboard);
        });
        const updateSettings = (settings) => {
            dashboardRenderFactory.defaultRenderOnSave = settings.get('renderOnSave')
                .composite;
        };
        if (settingRegistry) {
            Promise.all([settingRegistry.load(dashboardPlugin.id), app.restored])
                .then(([settings]) => {
                updateSettings(settings);
                settings.changed.connect(updateSettings);
            })
                .catch((reason) => {
                console.error(reason.message);
            });
        }
        commands.addCommand(CommandIDs.dashboardRender, {
            label: 'Dashboard',
            execute: async (args) => {
                const notebookPanel = getCurrentNotebookPanel(args);
                let context;
                if (notebookPanel) {
                    context = notebookPanel.context;
                    await context.save();
                    commands.execute('docmanager:open', {
                        path: context.path,
                        factory: DASHBOARD_WIDGET_FACTORY,
                        options: {
                            mode: 'split-right'
                        }
                    });
                }
            },
            isEnabled
        });
        commands.addCommand(CommandIDs.dashboardOpen, {
            label: 'Open with Dashboard',
            execute: async (args) => {
                const notebookPanel = getCurrentNotebookPanel(args);
                if (!notebookPanel) {
                    return;
                }
                await notebookPanel.context.save();
            },
            isEnabled
        });
        if (palette) {
            const category = 'Notebook Operations';
            [CommandIDs.dashboardRender, CommandIDs.dashboardOpen].forEach(command => {
                palette.addItem({ command, category });
            });
        }
        if (menu) {
            menu.viewMenu.addGroup([
                {
                    command: CommandIDs.dashboardRender
                },
                {
                    command: CommandIDs.dashboardOpen
                }
            ], 1000);
        }
        const dashboardRenderButton = new DashboardButton(commands);
        //
        docRegistry.addWidgetFactory(dashboardRenderFactory);
        docRegistry.addWidgetExtension('Notebook', dashboardRenderButton);
        //
        return tracker;
    }
};
export default dashboardPlugin;
//# sourceMappingURL=plugin.js.map